package s3browser

import (
	"context"
	"fmt"

	"github.com/minio/minio-go/v7"
)

func deleteMutation(ctx context.Context, id string) error {
	s3Client, ok := ctx.Value("s3Client").(*minio.Client)

	if !ok {
		return fmt.Errorf("Failed to get s3Client from context")
	}

	// TODO: it is posible to remove multiple objects with a single call.
	// Is it better to batch this?
	return s3Client.RemoveObject(ctx, bucketName, id, minio.RemoveObjectOptions{})
}

func copyMutation(ctx context.Context, src, dest string) (*File, error) {
	s3Client, ok := ctx.Value("s3Client").(*minio.Client)

	if !ok {
		return nil, fmt.Errorf("Failed to get s3Client from context")
	}

	info, err := s3Client.CopyObject(ctx, minio.CopyDestOptions{
		Bucket: bucketName,
		Object: dest,
	}, minio.CopySrcOptions{
		Bucket: bucketName,
		Object: src,
	})

	if err != nil {
		return nil, err
	}

	return &File{
		ID: info.Key,
	}, nil

}

func moveMutation(ctx context.Context, src, dest string) (*File, error) {
	s3Client, ok := ctx.Value("s3Client").(*minio.Client)

	if !ok {
		return nil, fmt.Errorf("Failed to get s3Client from context")
	}

	// There is no (spoon) move. Only copy and delete
	info, err := s3Client.CopyObject(ctx, minio.CopyDestOptions{
		Bucket: bucketName,
		Object: dest,
	}, minio.CopySrcOptions{
		Bucket: bucketName,
		Object: src,
	})

	if err != nil {
		return nil, err
	}

	err = deleteMutation(ctx, src)

	if err != nil {
		return nil, err
	}

	return &File{
		ID: info.Key,
	}, nil

}
